using HermesSocketLibrary.db;
using HermesSocketServer.Models;
using ILogger = Serilog.ILogger;

namespace HermesSocketServer.Requests
{
    public class CreateTTSUser : IRequest
    {
        public string Name => "create_tts_user";
        public string[] RequiredKeys => ["chatter", "voice"];
        private Database _database;
        private readonly ServerConfiguration _configuration;
        private ILogger _logger;

        public CreateTTSUser(Database database, ServerConfiguration configuration, ILogger logger)
        {
            _database = database;
            _configuration = configuration;
            _logger = logger;
        }

        public async Task<RequestResult> Grant(Channel channel, IDictionary<string, object> data)
        {
            if (!long.TryParse(data["chatter"].ToString(), out long chatterId))
                return RequestResult.Failed("Invalid Twitch user id");
            
            data["user"] = channel.Id;
            data["voice"] = data["voice"].ToString()!;

            var check = await _database.ExecuteScalar("SELECT state FROM \"TtsVoiceState\" WHERE \"userId\" = @user AND \"ttsVoiceId\" = @voice", data) ?? false;
            if ((check is not bool state || !state) && chatterId != _configuration.Tts.OwnerId)
                return RequestResult.Failed("Voice is disabled on this channel.");

            bool result = channel.Chatters.Set(chatterId.ToString(), new ChatterVoice()
            {
                UserId = channel.Id,
                ChatterId = chatterId,
                VoiceId = data["voice"].ToString()!
            });

            if (result)
            {
                _logger.Information($"Selected a tts voice [voice: {data["voice"]}] for user [chatter: {data["chatter"]}] in channel [channel: {data["user"]}]");
                return RequestResult.Successful(null);
            }
            return RequestResult.Failed("Something went wrong when updating the cache.");
        }
    }
}