using HermesSocketLibrary.Requests.Messages;
using HermesSocketServer.Models;
using HermesSocketServer.Store;
using ILogger = Serilog.ILogger;

namespace HermesSocketServer.Requests
{
    public class DeleteTTSVoice : IRequest
    {
        public string Name => "delete_tts_voice";
        public string[] RequiredKeys => ["voice"];
        private IStore<string, TTSVoice> _voices;
        private ILogger _logger;

        public DeleteTTSVoice(VoiceStore voices, ILogger logger)
        {
            _voices = voices;
            _logger = logger;
        }

        public Task<RequestResult> Grant(Channel channel, IDictionary<string, object> data)
        {
            string voiceId = data["voice"].ToString()!;
            var result = _voices.Remove(voiceId);

            if (result)
            {
                _logger.Information($"Deleted a voice by id [voice id: {voiceId}]");
                return Task.FromResult(RequestResult.Successful(null));
            }

            _logger.Warning($"Voice ID does not exist [id: {voiceId}]");
            return Task.FromResult(RequestResult.Failed("Voice ID does not exist."));
        }
    }
}