using HermesSocketLibrary.db;
using HermesSocketLibrary.Requests.Messages;
using HermesSocketServer.Store.Internal;

namespace HermesSocketServer.Store
{
    public class TTSFilterStore : AutoSavedStore<string, TTSWordFilter>
    {
        private readonly string _userId;
        private readonly Database _database;
        private readonly Serilog.ILogger _logger;


        public TTSFilterStore(string userId, DatabaseTable table, Database database, Serilog.ILogger logger)
            : base(table, database, logger)
        {
            _userId = userId;
            _database = database;
            _logger = logger;
        }

        public override async Task Load()
        {
            var data = new Dictionary<string, object>() { { "user", _userId } };
            string sql = $"SELECT id, search, replace FROM \"TtsWordFilter\" WHERE \"userId\" = @user";
            await _database.Execute(sql, data, (reader) =>
            {
                var id = reader.GetString(0);
                _store.Add(id.ToString(), new TTSWordFilter()
                {
                    Id = id,
                    Search = reader.GetString(1),
                    Replace = reader.GetString(2)
                });
            });
            _logger.Information($"Loaded {_store.Count} TTS chatter voices from database.");
        }

        protected override void OnInitialAdd(string key, TTSWordFilter value)
        {
        }

        protected override void OnInitialModify(string key, TTSWordFilter value)
        {
        }

        protected override void OnPostRemove(string key, TTSWordFilter value)
        {
        }
    }
}